<?php
/*
* @package brutal_cache
* @version 0.5
*/

/*

Plugin Name: Brutal Cache

Plugin URI: http://sateffen.bplaced.net

Description: Dieses Plugin gives total control over caching

Author: Steffen Wendt

Version: 0.5

Author URI: http://sateffen.bplaced.net

*/

//constant for security reasons
define('BRUTAL_CACHE_INSIDE', 'SAW_INSIDE');

//init the brutal cache programm
brutal_cache::init();
//test wheather the function "register_deactivation_hook" exists to prevent errors
if ( function_exists('register_deactivation_hook') ) {
    //hook the deactivation caller to the deactivation hook
    register_deactivation_hook(__FILE__,'brutal_cache::deactivate_caller');
}

/*
if ( function_exists('register_activation_hook') ) {
    register_activation_hook(__FILE__,'brutal_cache::activate_caller');
}
*/

//this is the Brutal cache programm itself.
class brutal_cache {

    //Set the needed variables
    private static $instanz = NULL;
    private $config = array();
    private $BRUTAL_CACHE_DIR = NULL;
    private $create_new_cache = false;

    /*
     * @action: this initialisizes the plugin in singleton pattering
     *
     * @return: the instance
     */
    public static function init() {
        if (self::$instanz === NULL) {
            self::$instanz = new self;
        }
        return self::$instanz;
    }

    /*
     * @action: do nothing, only securing the singleton pattering
     *
     * @return: void
     */

    private function  __clone() {
        //Empty to save the singleton pattering
    }

    /*
     * @action: is constructor sets the BRUTAL_CACHE_DIR and loads the configuration. if the configuration
     *          doesn't exist, the brutal cache programm won't be executed
     *
     * @return: void
     */

    private function __construct() {
        $this->BRUTAL_CACHE_DIR = str_replace( '\\', '/', dirname( __FILE__ ) );
        if ( file_exists( $this->BRUTAL_CACHE_DIR.'/brutal_cache_config.php' ) ) {
            include( $this->BRUTAL_CACHE_DIR.'/brutal_cache_config.php' );
            $this->config = ( is_array( $bcp_config ) ) ? $bcp_config : NULL;
            unset ( $bcp_config );
        } else {
            return;
        }
        
        if ( $this->config['brutal_mode'] == '1' ) {
            $this->controler();
        } else {
            switch ( $this->config['hook_event'] ) {
                case 1: $hook_event = 'plugins_loaded'; break;
                case 2: $hook_event = 'sanitize_comment_cookies'; break;
                case 3: $hook_event = 'setup_theme'; break;
                default: $hook_event = 'plugins_loaded'; break;
            }
            if ( function_exists('add_action') && function_exists('add_filter') ) {
                add_filter( 'show_admin_bar', '__return_false' );
                add_action( $hook_event, 'brutal_cache::controler_caller', $this->config['hook_prioritat'] );
            }
        }      
    }

    /*
     * @action: this function deletes all cachefiles
     *
     * @return: void
     */

    private function delete_cache($file=false) {
        if (!$file) {
            $h = opendir( $this->BRUTAL_CACHE_DIR.'/cache/' );
            while ( $f = readdir( $h ) ) {
                if ( $f != '.' && $f != '..' && $f != 'index.php' ) {
                    unlink( $this->BRUTAL_CACHE_DIR.'/cache/'.$f );
                }
            }
            closedir( $h );
        } else {
            unlink( $this->BRUTAL_CACHE_DIR.'/cache/'.$file );
        }
    }

    /*
     * action: this is the main core of the programm. it will check which configuration is set and
     *         will serve the cache if it exists, or set the hooks to create the new cachefile
     *
     * @return: void
     */

    private function init_cache() {        
        $cache_file = $this->BRUTAL_CACHE_DIR.'/cache/'.$this->get_cache_filename().'.cache';
        $cache_file_timestamp = @filemtime( $cache_file );

        //Prüfen ob ein Submit gedrückt wurde und für den fall der fälle die Cache neu generieren
        if ( isset( $_POST['submit'] ) && !empty( $_POST['submit'] ) ) {
            $this->create_new_cache = true;
            add_action( 'get_header', 'brutal_cache::record_start', '20' );
            add_action( 'wp_print_footer_scripts', 'brutal_cache::destruct_caller', '20' );
            return;
        }

        if ( $this->config['use_user_stored'] == 1 && file_exists($cache_file) ) {
            //Cache laden wenn Files nicht Verändert
            $gmt_mtime = gmdate( 'r', $cache_file_timestamp );
            header( 'ETag: "'.md5( $cache_file_timestamp.$cache_file ).'"' );
            if( @isset( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) || @isset( $_SERVER['HTTP_IF_NONE_MATCH'] ) ) {
                if ( $_SERVER['HTTP_IF_MODIFIED_SINCE'] == $gmt_mtime || str_replace( '"', '', stripslashes( $_SERVER['HTTP_IF_NONE_MATCH']) ) == md5( $cache_file_timestamp.$cache_file ) ) {
                    header( 'HTTP/1.1 304 Not Modified' );
                    exit();
                }
            }
            header( 'Last-Modified: '.$gmt_mtime );
            header( 'Cache-Control: public' );
        }

        if ( $this->config['timeval_active'] == 1 ) {
            if ( file_exists( $cache_file ) && ( $cache_file_timestamp + $this->config['timeval_time'] > time() ) ) {
                $cache = file_get_contents( $cache_file );
                if ( !empty( $cache ) ) {
                    exit( $cache );
                } else {
                    $this->create_new_cache = true;
                }
            } else {
                $this->create_new_cache = true;
            }
        } else {
            if ( file_exists( $cache_file ) ) {
                $cache = file_get_contents( $cache_file );
                if ( !empty( $cache ) ) {
                    exit( $cache );
                } else {
                    $this->create_new_cache = true;
                }
            } else {
                }
                $this->create_new_cache = true;
            }
        if ( $this->create_new_cache ) {
            add_action( 'get_header', 'brutal_cache::record_start', '20' );
            add_action( 'wp_print_footer_scripts', 'brutal_cache::destruct_caller', '20' );
        }
    }

    /*
     * @action: starts the output buffering for the cache file
     *
     * @return: void
     */

    public static function record_start() {
        ob_start();
    }

    /*
     * action: This function is called when the cachefile should be written. It deletes
     *         the comments, the multiple whitespace sequences and the whitespaces before
     *         and after tags, so it's shorting the code and minifies the cachefile
     *
     * @return: string
     */

    private function minify_content($content) {
        $search = array(
            '/\>[^\S ]+/s', //strip whitespaces after tags, except space
            '/[^\S ]+\</s', //strip whitespaces before tags, except space
            '/<!--.+?-->/s' //delete comments
            );
        $replace = array(
            '>',
            '<',
            ''
            );
        return preg_replace($search, $replace, $content);
    }

    /*
     * @action: calles the destruct function
     *
     * @return: void
     */

    public static function destruct_caller() {
        self::$instanz->destruct();
    }

    /*
     * @action: watch out! this is NOT the destructor! this function is called at the end of the
     *          brutal cache programm and creates the cachefiles
     *
     * @return: void
     */

    private function destruct() {
        if ( $this->create_new_cache === true ) {
            $cache = ob_get_clean();
            if ( $this->config['reduce_http'] == 1 ) {
                $cache = $this->reduce_http_requests($cache);
            }
            if ( $this->config['minify_cache'] == 1 ) {
                $cache = $this->minify_content($cache);
            }
            file_put_contents( $this->BRUTAL_CACHE_DIR.'/cache/'.$this->get_cache_filename().'.cache', $cache );
            echo $cache;
        }
    }

    /*
     * @action: calles the deactivate funktion
     *
     * @return: void
     */

    public static function deactivate_caller() {
        self::$instanz->deactivate();
    }

    /*
     * @action: calles the activate function
     *
     * @return: void
     */

    public static function activate_caller() {
        self::$instanz->activate();
    }

    /*
     * @action: this function is called when the the plugin is deactivated. it will deactivate the brutal mode
     *          to secure the further working of wordpress after deactivation
     *
     * @return: void
     */

    private function deactivate() {
        $this->deactivate_brutal_mode();
        $this->config['brutal_mode'] = 0 ;
        $this->save_config_to_file();
    }

    /*
     * @action: this is called when the plugin is activated and set the chmod for the used files
     *
     * @return: void
     */

    private function activate() {
        $this->set_plugin_chmod();
    }

    /*
     * @action: caller for the controler
     *
     * @return: void
     */

    public static function controler_caller () {
        self::$instanz->controler();
    }

    /*
     * @action this part controls the actions by the URI to decide what to do
     *
     * @return: void
     */

    private function controler() {
        if ( strpos( $_SERVER['REQUEST_URI'],'/wp-admin/' ) !== false ) {
            add_action( 'admin_menu', 'brutal_cache::admin_caller' );
            if ( $this->config['delete_by_admin_call'] ) {
                $this->delete_cache();
            }
        } elseif ( strpos( $_SERVER['REQUEST_URI'],'/wp-login.php' ) !== false ) {
        } else {
            $this->init_cache();
        }
    }

    /*
     * @action: This function add's the Brutal Cache configuration menue link to the admin panel
     *
     * @return: void
     */

    public static function admin_caller() {
        add_options_page( 'Brutal Cache Options', 'Brutal Cache', 'manage_options', 'brutal-cache', 'brutal_cache::admin_menu_caller' );
    }

    /*
     * @action: this calles the admin page out of the admin panel
     *
     * @return: void
     */

    public static function admin_menu_caller() {
        self::$instanz->admin_page();
    }

    /*
     * @action: this function sets the needed CHMODs to the used files
     *
     * @return: void
     */

    private function set_plugin_chmod() {
        //set chmod for the config file
        if ( !( decoct( fileperms( $this->BRUTAL_CACHE_DIR.'/brutal_cache_config.php' ) ) == 100700 ) ) {
            @chmod($this->BRUTAL_CACHE_DIR.'/brutal_cache_config.php', 0700);
        }
        
        //set chmod for the brutal mode file
        if ( !( decoct( fileperms( $this->BRUTAL_CACHE_DIR.'/brutal_cache_brutal_mode.php' ) ) == 100700 ) ) {
            @chmod($this->BRUTAL_CACHE_DIR.'/brutal_cache_brutal_mode.php', 0700);
        }

        //set chmod for cache directory
        if ( !( decoct( fileperms( $this->BRUTAL_CACHE_DIR.'/cache' ) ) == 100700 ) ) {
            @chmod($this->BRUTAL_CACHE_DIR.'/cache', 0700);
        }

        //set chmod for main file
        if ( !( decoct( fileperms( __FILE__ ) ) == 100700 ) ) {
            @chmod(__FILE__, 0700);
        }
    }

    /*
     * @action: saves the new config to the configuration file
     *
     * @return: void
     */

    private function save_config_to_file() {
        ob_start();
        var_export( $this->config );
        file_put_contents( $this->BRUTAL_CACHE_DIR.'/brutal_cache_config.php', "<?php \n if (constant('BRUTAL_CACHE_INSIDE') !== 'SAW_INSIDE' ) { die('No Access'); } \n ".'$bcp_config='.ob_get_clean()."; \n ?>" );
    }

    /*
     * @action: this function activates the brutal mode
     *
     * @return: bool
     */

    private function activate_brutal_mode() {
        chdir( '..' );
        $index_file_contents = file_get_contents( getcwd().'/index.php' );

        if (preg_match('/brutal_cache_brutal_mode.php/i', $index_file_contents) ) {
            return true;
        }

        $index_file_contents = str_replace( '<?php', '<?php '."\n".'@include_once("'.$this->BRUTAL_CACHE_DIR.'/brutal_cache_brutal_mode.php");',$index_file_contents );

        if ( file_put_contents( getcwd().'/index.php',$index_file_contents ) ) {
            return true;
        } else {
            return false;
        }
    }

    /*
     * @action: this function deactivates the brutal mode
     *
     * @return: bool
     */

    private function deactivate_brutal_mode() {
        chdir('..');
        $index_file_contents = file_get_contents( getcwd().'/index.php' );
        $index_file_contents = str_replace( "\n".'@include_once("'.$this->BRUTAL_CACHE_DIR.'/brutal_cache_brutal_mode.php");', '', $index_file_contents );

        if ( file_put_contents( getcwd().'/index.php',$index_file_contents ) ) {
            return true;
        } else {
            return false;
        }
    }

    /*
     * @action: this will call the brutal cache configuration page in the right language to use. also it
     *          sets the chmod which is needed so prevent errors of the right system
     *
     * @return: void
     */

    private function admin_page() {
        //set notice to false to make it empty
        $notice = false;

        //Set chmod for plugin files
        $this->set_plugin_chmod();

        //choose which language to use
        switch ( WPLANG ) {
            case 'de_DE':
                include_once( $this->BRUTAL_CACHE_DIR.'/adminmenu/adminmenu_de.php' );
                break;
            case 'en_EN':
                include_once( $this->BRUTAL_CACHE_DIR.'/adminmenu/adminmenu_en.php' );
                break;
            default:
                include_once( $this->BRUTAL_CACHE_DIR.'/adminmenu/adminmenu_en.php' );
                break;
        }
    }

    /*
     * @action: This function checks the input string/text for CSS and JS links and preloads it and
     *          writes it to the DOMDocument. After replacing it all the new validated DOMDocument is
     *          given back
     *
     * @return: string/text
     */

    private function reduce_http_requests($html) {
        $dom = new DOMDocument;
        @$dom->loadHTML($html);
        $scripts = $dom->getElementsByTagName('script'); //select scripts
        $styles = $dom->getElementsByTagName('link'); //select styles

        foreach ( $scripts as $script ) {
            if ( $script->hasAttribute('src') ) {
                $script_file = @file_get_contents( $script->getAttribute('src') );
                if ( $script_file !== false && !empty( $script_file ) ) {
                    $newnode = $dom->createElement( 'script', $script_file );
                    $newnode->setAttribute( 'type','text/javascript' );
                    $newnode->setAttribute( 'language', 'javascript' );
                    $test = $script->parentNode->replaceChild( $newnode , $script );
                }
            }
        } //end script foreach
        
        foreach ( $styles as $style ) {
            if ( $style->hasAttribute('href') && $style->hasAttribute('rel') && $style->getAttribute('type') == 'text/css' ) {
                $sytle_file = @file_get_contents( $style->getAttribute('href') );
                if ( $sytle_file !== false && !empty( $style_file ) ) {
                    $newnode = $dom->createElement( 'style', $style_file );
                    $newnode->setAttribute( 'type', 'text/css' );
                    $style->parentNode->replaceChild( $newnode , $style );
                }
            }
        } //end style foreach
        
        return $dom->saveHTML();
    }

    /*
     * @action: This function creates the cachefile name with is asked for and returns it
     *
     * @return: string
     */

    private function get_cache_filename() {
        if ($_SERVER['REQUEST_URI'] == '/') {
            return 'home';
        } else {
            return str_replace('/', '.', $_SERVER['REQUEST_URI']);
        }
    }

}

?>